#!/bin/sh

## live-build(7) - System Build Scripts
## Copyright (C) 2016-2020 The Debian Live team
## Copyright (C) 2006-2015 Daniel Baumann <mail@daniel-baumann.ch>
##
## This program comes with ABSOLUTELY NO WARRANTY; for details see COPYING.
## This is free software, and you are welcome to redistribute it
## under certain conditions; see COPYING for details.


NO_COLOR="\033[0m"

BOLD="\033[1m"
UNDERSCORE="\033[4m"
BLINK="\033[5m"

BLACK="\033[0;30m"
DARK_GRAY="\033[1;30m"

RED="\033[0;31m"
LIGHT_RED="\033[1;31m"

GREEN="\033[0;32m"
LIGHT_GREEN="\033[1;32m"

BROWN="\033[0;33m"
YELLOW="\033[1;33m"

BLUE="\033[0;34m"
LIGHT_BLUE="\033[1;34m"

PURPLE="\033[0;35m"
LIGHT_PURPLE="\033[1;35m"

CYAN="\033[0;36m"
LIGHT_CYAN="\033[1;36m"

GRAY="\033[0;37m"
WHITE="\033[1;37m"
